<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\FuncionariosService;

class FuncionariosController extends FMCrud{
    protected $dataSourceName = 'fcm_checklistagend';
    protected $FuncionariosService;
    protected $environment;

    public function __construct($crudService, $environment, FuncionariosService $FuncionariosService) {
        parent::__construct($crudService, $environment);
        $this->FuncionariosService = $FuncionariosService;
        $this->environment       = $environment;
    }

    public function getShift(DTO\Request $request, DTO\Response $response) {

        $variavel = $this->FuncionariosService->searchShift();
        $dtsVal = new DataSet('shift', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getOffice(DTO\Request $request, DTO\Response $response) {

        $variavel = $this->FuncionariosService->searchOffice();
        $dtsVal = new DataSet('office', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getTasks(DTO\Request $request, DTO\Response $response) {

        $variavel = $this->FuncionariosService->searchTasks();
        $dtsVal = new DataSet('tasks', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getOccupation(DTO\Request $request, DTO\Response $response) {

        $variavel = $this->FuncionariosService->searchOccupation();
        $dtsVal = new DataSet('occupation', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getSector(DTO\Request $request, DTO\Response $response) {

        $variavel = $this->FuncionariosService->searchSector();
        $dtsVal = new DataSet('sector', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getDataBaseCards(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();

        $variavel = $this->FuncionariosService->dataBaseCards($parametro);
        $dtsVal = new DataSet('dataBaseCards', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getDataBaseTaksCards(DTO\Request $request, DTO\Response $response) {

        $parametro = $request->getParameters();
        $nrfuncionario = $parametro['employeesTasks']['NRFUNCIONARIO'];

        $variavel = $this->FuncionariosService->dataBaseTaskCards($nrfuncionario);
        $dtsVal = new DataSet('dataBaseCards', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function dataBaseNewEmployee(DTO\Request $request, DTO\Response $response) {

        $parametro = $request->getParameters();

        $nmfuncionario      = $parametro['NewEmployees']['NMFUNCIONARIO'];
        $nrrgfuncionario    = $parametro['NewEmployees']['NRRGFUNCIONARIO'];
        $nrcpffuncionario   = $parametro['NewEmployees']['NRCPFFUNCIONARIO'];
        $dsemailfuncionario = $parametro['NewEmployees']['DSEMAILFUNCIONARIO'];
        $nrtelfuncionario   = $parametro['NewEmployees']['NRTELFUNCIONARIO'];
        $hrentradapadr      = $parametro['NewEmployees']['HRENTRADAPADR'];
        $hrsaidapadr        = $parametro['NewEmployees']['HRSAIDAPADR'];
        $idturnofuncionario = $parametro['NewEmployees']['IDTURNOFUNCIONARIO'];
        $nrorg              = $this->environment->getNrOrg();
        $operador           = $parametro['NewEmployees']['OPERADOR'];
        $nrhabilidade       = $parametro['NewEmployees']['NRHABILIDADE'];


        $variavel = $this->FuncionariosService->dataBaseNewEmployee($nmfuncionario, $nrrgfuncionario, $nrcpffuncionario, $dsemailfuncionario, $nrtelfuncionario, $hrentradapadr, $hrsaidapadr, $idturnofuncionario, $nrorg, $operador, $nrhabilidade);

        $dtsVal = new DataSet('newEmployee', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function deleteEmployeeDataBase(DTO\Request $request, DTO\Response $response) {

        $parametro = $request->getParameters();

        $nrfuncionario      = $parametro['deleteEmployee']['NRFUNCIONARIO'];
        $nrrelacfunchab     = $parametro['deleteEmployee']['NRRELACFUNCHAB'];
        $nrorg              = $this->environment->getNrOrg();

        $variavel = $this->FuncionariosService->deleteEmployeeDataBase($nrfuncionario, $nrorg, $nrrelacfunchab);
        $dtsVal = new DataSet('deleteEmployee', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function updateEmployee(DTO\Request $request, DTO\Response $response) {

        $parametro = $request->getParameters();

        $nmfuncionario      = $parametro['updateEmployee']['NMFUNCIONARIO'];
        $nrrgfuncionario    = $parametro['updateEmployee']['NRRGFUNCIONARIO'];
        $nrcpffuncionario   = $parametro['updateEmployee']['NRCPFFUNCIONARIO'];
        $dsemailfuncionario = $parametro['updateEmployee']['DSEMAILFUNCIONARIO'];
        $nrtelfuncionario   = $parametro['updateEmployee']['NRTELFUNCIONARIO'];
        $hrentradapadr      = $parametro['updateEmployee']['HRENTRADAPADR'];
        $hrsaidapadr        = $parametro['updateEmployee']['HRSAIDAPADR'];
        $idturnofuncionario = $parametro['updateEmployee']['IDTURNOFUNCIONARIO'];
        $nrhabilidade       = $parametro['updateEmployee']['NRHABILIDADE'];
        $nrfuncionario      = $parametro['updateEmployee']['NRFUNCIONARIO'];
        $nrorg              = $this->environment->getNrOrg();

        $variavel = $this->FuncionariosService->updateEmployeeDataBase($nmfuncionario, $nrrgfuncionario, $nrcpffuncionario, $dsemailfuncionario, $nrtelfuncionario, $hrentradapadr, $hrsaidapadr, $idturnofuncionario, $nrhabilidade, $nrfuncionario, $nrorg);

        $dtsVal = new DataSet('updateEmployee', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getInfoEmployee(DTO\Request $request, DTO\Response $response) {

        $parametro = $request->getParameters();
        $nrfuncionario      = $parametro['getInfo']['NRFUNCIONARIO'];
        $nrrelacfunchab     = $parametro['getInfo']['NRRELACFUNCHAB'];
        $nrorg              = $this->environment->getNrOrg();

        $variavel = $this->FuncionariosService->getInfoEmployee($nrfuncionario, $nrorg, $nrrelacfunchab);
        $dtsVal = new DataSet('getInfomation', $variavel);
        $response->addDataSet($dtsVal);
    }
}
?>