import {
  Metadata, Http, IDictionary, IEventParam,
} from '@zeedhi/core';
import {
  Select, Date as DateInput, Modal, ModalService, ITabsBeforeChangeEventParam, Button, AlertService,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class ConformidadesController {
  public dataSelects: IDictionary = {
    selectLocal: [],
    selectTipoLocal: [],
    selectSetor: [],
  };

  public Gestores: IDictionary = {};

  constructor() {
    this.requestGestores('/NaoConformidades/Gestor')
      .then((gestores: any) => {
        this.Gestores = gestores;
      })
      .catch((err) => {
        AlertService.show({
          name: 'basic-alert',
          top: true,
          left: false,
          color: 'error',
          text: err,
          timeout: 3000,
        });
      });
  }

  public setData(nome: string, cod: number) {
    return {
      name: nome,
      value: cod,
    };
  }

  public getDadosCard() {
    Metadata.getInstance<IterableComponentRender>('cardNaoConformidade').datasource.data = [];
    Http.post('/getDadosCard', this.dataFilter).then((response: IDictionary) => {
      const dadosCard = response.data.dataset.dados;
      Object.keys(dadosCard).forEach((dados: string) => {
        Metadata.getInstance<IterableComponentRender>('cardNaoConformidade').datasource.data.push({
          LOCALCLIENTE: dados,
          DADOSFILHO: dadosCard[dados],
        });
      });
    });
  }

  public setConformidades({ component }:{ component:IterableComponentRender }) {
    const parentIterable = Metadata.getInstance<IterableComponentRender>('cardNaoConformidade');
    const id = component.name.split('_').pop();
    const parentIndex = parentIterable.datasource.data.findIndex((row: IDictionary) => row.LOCALCLIENTE === id);
    parentIterable.datasource.data[parentIndex].DADOSFILHO.forEach((local: string) => {
      component.datasource.data.push({
        CRITERIO: local,
      });
    });
  }

  public getDadosCardPrincipais() {
    const iterable = Metadata.getInstance<IterableComponentRender>('cardPrincipaisNaoConformidade');
    iterable.datasource.data = [];

    Http.post('/getDadosCardPrincipais', this.dataFilterPeriodo).then((response: IDictionary) => {
      response.data.dataset.dadosCardPrincipais.forEach((dados: IDictionary) => {
        iterable.datasource.data.push({
          NAME: dados.NMCRITERIOTARGERCHK,
          QUANTIDADE: dados.QUANTIDADE,
        });
      });
    });
  }

  public getDataInfo({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public refreshData({ component }: { component: Select }) {
    component.datasource.get();
  }

  public setDadosLocal({ component }: any) {
    Http.get('/getPlace').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.place);
    });
  }

  public setDadosTipoLocal({ component }: any) {
    Http.get('/getTipoLocal').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.tipoLocal);
    });
  }

  public setDadosSetor({ component }: any) {
    Http.get('/getSetor').then((response: IDictionary) => {
      component.datasource.updateData(response.data.dataset.setor);
    });
  }

  get getLocal() { // pegar o select Local
    return Metadata.getInstance<Select>('selectLocal').value || '';
  }

  get getTipoLocal() { // pegar o select tipo local
    return Metadata.getInstance<Select>('selectTipoLocal').value || '';
  }

  get getSetor() { // pegar o select do setor
    return Metadata.getInstance<Select>('selectSetor').value || '';
  }

  get getDataDe() {
    let dataDe = Metadata.getInstance<DateInput>('dateDe').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    return dataDe;
  }

  get getDataAte() {
    let dataAte = Metadata.getInstance<DateInput>('dateAte').value;
    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    return dataAte;
  }

  get getDataFiltroDe() {
    let dataDe = Metadata.getInstance<DateInput>('dataDe').value;
    if (dataDe) {
      dataDe = (dataDe.length === 10) ? dataDe : null;
    } else {
      dataDe = null;
    }
    return dataDe;
  }

  get getDataFiltroAte() {
    let dataAte = Metadata.getInstance<DateInput>('dataAte').value;

    if (dataAte) {
      dataAte = (dataAte.length === 10) ? dataAte : null;
    } else {
      dataAte = null;
    }
    return dataAte;
  }

  private dataFilterPeriodo: IDictionary = {
    selectDataDe: '',
    selectDataAte: '',
  };

  private dataFilter: IDictionary = {
    selectLocal: '',
    selectTipoLocal: '',
    selectSetor: '',
    selectDataDe: '',
    selectDataAte: '',
  };

  public filter() {
    this.dataFilter.selectLocal = this.getLocal;
    this.dataFilter.selectTipoLocal = this.getTipoLocal;
    this.dataFilter.selectSetor = this.getSetor;
    this.dataFilter.selectDataDe = this.getDataDe;
    this.dataFilter.selectDataAte = this.getDataAte;
    this.getDadosCard();
    this.closeBasicModal();
  }

  public filterPeriodo() {
    this.dataFilterPeriodo.selectDataDe = this.getDataFiltroDe;
    this.dataFilterPeriodo.selectDataAte = this.getDataFiltroAte;
    this.getDadosCardPrincipais();
    this.closeModalPrincipaisConformidades();
  }

  private basicModal!: Modal;

  private basicModalConformidades!: Modal;

  public async createBasicUsageModal() {
    this.basicModal = await ModalService.createFromJson('/filtros', true);
    this.basicModalConformidades = await ModalService.createFromJson('/modalFiltroPrincipaisConformidades', true);
  }

  public destroyBasicUsageModal() {
    this.basicModal.destroy();
  }

  public openBasicModal() {
    this.basicModal.show();
  }

  public openModalPrincipaisConformidades() {
    this.basicModalConformidades.show();
  }

  public closeModalPrincipaisConformidades() {
    this.basicModalConformidades.hide();
  }

  public closeBasicModal() {
    this.basicModal.hide();
  }

  public setIndexTab({ component, tabIndex }: ITabsBeforeChangeEventParam) {
    const name = component.tabs[tabIndex].tabTitle;
    const buttonObject = Metadata.getInstance<Button>('iconFiltro');
    if (name === 'Principais Não Conformidades') {
      buttonObject.events = {
        click: () => this.openModalPrincipaisConformidades(),
      };
    } else {
      buttonObject.events = {
        click: () => this.openBasicModal(),
      };
    }
  }

  public requestGestores(url: string) {
    return new Promise((resolve, reject) => {
      // const url = '/NaoConformidades/Gestor?' + strFilters;
      Http.get(url)
        .then((response: IDictionary) => {
          const promiseGestores: Array<Promise<any>> = [];
          const data: Array<IDictionary> = response.data.dataset.conformidadesGestor;
          const gestores: IDictionary = {};
          const auxObject: IDictionary = {};
          data.forEach((item: IDictionary) => {
            // checa se o funcionario tem um func pai válido. se não tiver, o item é ignorado
            if (item.NRFUNCIONARIOPAI !== null) {
              const nrfuncPaiString: string = item.NRFUNCIONARIOPAI.toString();
              // checa se o gestor já foi inserido na lista de gestores
              // se não estiver na lista, vai ser inserido e criado as propriedades
              if (!(nrfuncPaiString in gestores)) {
                gestores[nrfuncPaiString] = {};
                gestores[nrfuncPaiString].TOTALNAOCONFORMIDADES = 0;
                gestores[nrfuncPaiString].FUNCIONARIOS = {};
                // criando o array de promises
                promiseGestores.push(Http.get(`/Employee?NRFUNCIONARIO=${nrfuncPaiString}`));
              }
              const nrFuncStr: string = item.NRFUNCIONARIO.toString();
              // verficando se o funcionario do item atual, ja foi registrado na lista do gestor atual
              // se não estiver na lista, cria todas as propriedades necessárias
              if (!(nrFuncStr in gestores[nrfuncPaiString].FUNCIONARIOS)) {
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr] = {};
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NMFUNCIONARIO = item.NMFUNCIONARIO;
                gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NAOCONFORMIDADES = [];
                // objeto auxiliar usado para formatar o texto final das nao conformidades
                auxObject[nrFuncStr] = {};
              }
              const criterioTarefa = item.DSCRITERIOTAREFA;
              // verifica se a tarefa atual já foi inserida no objeto auxiliar
              if (!(criterioTarefa in auxObject[nrFuncStr])) {
                auxObject[nrFuncStr][criterioTarefa] = 1;
              } else {
                auxObject[nrFuncStr][criterioTarefa] += 1;
              }
              gestores[nrfuncPaiString].TOTALNAOCONFORMIDADES += 1;
              // gestores[nrfuncPaiString].FUNCIONARIOS[nrFuncStr].NAOCONFORMIDADES.push(item.DSCRITERIOTAREFA);
            }
          });
          // iterando sobre as chaves de gestores
          Object.keys(gestores).forEach((nrGestor: string) => {
            // iterando sobre as chaves dos funcionarios do atual gestor
            Object.keys(gestores[nrGestor].FUNCIONARIOS).forEach((nrFunc) => {
              let totalNConformidades = 0;
              // iterando sobre as nao conformidades do funcionario atual
              // a chave do objeto, possui a nao conformidade, e dentro dessa chave, consta o valor de vezes que a nao conformidade se repete
              Object.keys(auxObject[nrFunc]).forEach((nConformidade) => {
                gestores[nrGestor].FUNCIONARIOS[nrFunc].NAOCONFORMIDADES.push(`${nConformidade} (${auxObject[nrFunc][nConformidade]}x)`);
                totalNConformidades += auxObject[nrFunc][nConformidade];
              });
              // inserindo atributo de total nao conformidades
              // útil para informar o numero de nao conformidades ao card
              gestores[nrGestor].FUNCIONARIOS[nrFunc].TOTALNAOCONFORMIDADES = totalNConformidades;
            });
          });
          // garantindo que vai executar de forma sincrona
          // essas promises servem para buscar o NOME do gestor no banco de dados
          Promise.all(promiseGestores)
            .then((respPromises: Array<IDictionary>) => {
              respPromises.forEach((responses: IDictionary) => {
                const gestor = responses.data.dataset.employee[0];
                const nrFuncString = gestor.NRFUNCIONARIO.toString();
                gestores[nrFuncString].NMGESTOR = gestor.NMFUNCIONARIO;
              });
              resolve(gestores);
            })
            .catch((err) => {
              AlertService.show({
                name: 'basic-alert',
                top: true,
                left: false,
                color: 'error',
                text: err,
                timeout: 3000,
              });
              reject(err);
            });
        })
        .catch((err) => {
          AlertService.show({
            name: 'basic-alert',
            top: true,
            left: false,
            color: 'error',
            text: err,
            timeout: 3000,
          });
          reject(err);
        });
    });
  }

  public updateDataSourceGestoresCard({ component }: { component: IterableComponentRender }) {
    Object.keys(this.Gestores).forEach((nrGestor) => {
      component.datasource.data.push({
        NMGESTOR: this.Gestores[nrGestor].NMGESTOR,
        NRFUNCIONARIO: nrGestor,
      });
    });
  }

  public updateDataSourceFuncionariosCard({ component }: { component: IterableComponentRender }) {
    Object.keys(this.Gestores).forEach((nrGestor) => {
      Object.keys(this.Gestores[nrGestor].FUNCIONARIOS).forEach((nrFuncionario) => {
        component.datasource.data.push({
          NMFUNC: this.Gestores[nrGestor].FUNCIONARIOS[nrFuncionario].NMFUNCIONARIO,
          NRFUNCIONARIO: nrFuncionario,
          NRGESTOR: nrGestor,
        });
      });
    });
  }

  public funcIterable({ component }: { component: IterableComponentRender }) {
    const nrGestor = component.name.split('-')[1];
    Object.keys(this.Gestores[nrGestor].FUNCIONARIOS).forEach((nrFunc) => {
      component.datasource.data.push({
        NRFUNCIONARIO: nrFunc,
        NMFUNCIONARIO: this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].NMFUNCIONARIO,
      });
    });
  }

  public updateDataSourceCriterios({ component }: { component: IterableComponentRender }) {
    const splitedName = component.name.split('_');
    const nrGestor = splitedName[1];
    const nrFunc = splitedName[2];
    this.Gestores[nrGestor].FUNCIONARIOS[nrFunc].NAOCONFORMIDADES.forEach((naoConformidade: string) => {
      component.datasource.data.push({
        criterio: naoConformidade,
      });
    });
  }
}
